( cd usr/lib ; rm -rf libleck.so )
( cd usr/lib ; ln -sf libleck.so.3.0.5 libleck.so )
( cd usr/lib ; rm -rf liblite.so.3 )
( cd usr/lib ; ln -sf liblite.so.3.0.5 liblite.so.3 )
( cd usr/lib ; rm -rf liblite.so )
( cd usr/lib ; ln -sf liblite.so.3.0.5 liblite.so )
( cd usr/lib ; rm -rf libleck.so.3 )
( cd usr/lib ; ln -sf libleck.so.3.0.5 libleck.so.3 )
